//
//  AccountPreferences.swift
//  Tusker
//
//  Created by Shadowfacts on 12/19/22.
//  Copyright © 2022 Shadowfacts. All rights reserved.
//

import Foundation
import CoreData
import Pachyderm

@objc(AccountPreferences)
public final class AccountPreferences: NSManagedObject {
    
    @nonobjc class func fetchRequest(account: LocalData.UserAccountInfo) -> NSFetchRequest<AccountPreferences> {
        let req = NSFetchRequest<AccountPreferences>(entityName: "AccountPreferences")
        req.predicate = NSPredicate(format: "accountID = %@", account.id)
        return req
    }
    
    @NSManaged public var accountID: String
    @NSManaged var pinnedTimelinesData: Data?
    
    @LazilyDecoding(from: \AccountPreferences.pinnedTimelinesData, fallback: [])
    var pinnedTimelines: [Timeline]
    
    static func `default`(account: LocalData.UserAccountInfo, context: NSManagedObjectContext) -> AccountPreferences {
        let prefs = AccountPreferences(context: context)
        prefs.accountID = account.id
        prefs.pinnedTimelines = [.home, .public(local: true), .public(local: false)]
        return prefs
    }
    
}
