//  PreferencesView.swift
//  Tusker
//
//  Created by Shadowfacts on 6/13/19.
//  Copyright © 2019 Shadowfacts. All rights reserved.
//

import SwiftUI
import TTTKit

struct PreferencesView: View {
    let mastodonController: MastodonController
    @ObservedObject var localData = LocalData.shared
    @State private var showingLogoutConfirmation = false
    
    init(mastodonController: MastodonController) {
        self.mastodonController = mastodonController
    }

    var body: some View {
        // workaround: the navigation view is provided by MyProfileTableViewController so that it can inject the Done button
//        NavigationView {
            List {
                Section(header: Text("Accounts")) {
                    ForEach(localData.accounts, id: \.accessToken) { (account) in
                        Button(action: {
                            NotificationCenter.default.post(name: .activateAccount, object: nil, userInfo: ["account": account])
                        }) {
                            HStack {
                                LocalAccountAvatarView(localAccountInfo: account)
                                VStack(alignment: .leading) {
                                    Text(verbatim: account.username)
                                        .foregroundColor(.primary)
                                    Text(verbatim: account.instanceURL.host!)
                                        .font(.caption)
                                        .foregroundColor(.primary)
                                }
                                Spacer()
                                if account == mastodonController.accountInfo! {
                                    Image(systemName: "checkmark")
                                        .renderingMode(.template)
                                        .foregroundColor(.secondary)
                                }
                            }
                        }.onDrag {
                            let activity = UserActivityManager.mainSceneActivity(accountID: account.id)
                            return NSItemProvider(object: activity)
                        }
                    }.onDelete { (indices: IndexSet) in
                        var indices = indices
                        var logoutFromCurrent = false
                        if let index = indices.first(where: { localData.accounts[$0] == mastodonController.accountInfo! }) {
                            logoutFromCurrent = true
                            indices.remove(index)
                        }
                        
                        indices.forEach { localData.removeAccount(localData.accounts[$0]) }
                        
                        if logoutFromCurrent {
                            self.logoutPressed()
                        }
                    }
                    
                    Button(action: {
                        NotificationCenter.default.post(name: .addAccount, object: nil)
                    }) {
                        Text("Add Account...")
                    }
                    if localData.getMostRecentAccount() != nil {
                        Button(action: {
                            self.showingLogoutConfirmation = true
                        }) {
                            Text("Logout from current")
                        }.alert(isPresented: $showingLogoutConfirmation) {
                            Alert(title: Text("Are you sure you want to logout?"), message: nil, primaryButton: .destructive(Text("Logout"), action: self.logoutPressed), secondaryButton: .cancel())
                        }
                    }
                }
                
                Section {
                    NavigationLink(destination: AppearancePrefsView()) {
                        Text("Appearance")
                    }
                    NavigationLink(destination: ComposingPrefsView()) {
                        Text("Composing")
                    }
                    NavigationLink(destination: MediaPrefsView()) {
                        Text("Media")
                    }
                    NavigationLink(destination: BehaviorPrefsView()) {
                        Text("Behavior")
                    }
                    NavigationLink(destination: WellnessPrefsView()) {
                        Text("Digital Wellness")
                    }
                    NavigationLink(destination: AdvancedPrefsView()) {
                        Text("Advanced")
                    }
                }
                
                Section {
                    NavigationLink("About") {
                        AboutView()
                    }
                    NavigationLink("Acknowledgements") {
                        AcknowledgementsView()
                    }
                }
            }
            .listStyle(InsetGroupedListStyle())
            .navigationBarTitle(Text("Preferences"), displayMode: .inline)
//        }
    }
    
    func logoutPressed() {
        NotificationCenter.default.post(name: .userLoggedOut, object: nil)
    }
}

//#if DEBUG
//struct PreferencesView_Previews : PreviewProvider {
//    static var previews: some View {
//        return PreferencesView()
//    }
//}
//#endif
